/*
 * MultiPane.c
 *
 * Routines for handling multi-pane dialog boxes. Design loosely based on some code I 
 * saw in Develop 23.
 *
 * MDT 3/11/96
 */

#include "MultiPane.h"

/* GetNewMPDialog
 * Gets a new MPDialog from the resource fork. An MPDialog is composed of the following 
 * resources: A DLOG of id dID, a dtl# of id dID. The DLOG is the master dialog box; 
 * the dialog that contains all items common to our sub-views. The dtl# contains a list
 * of DITLs composing our sub-panes. The format of the dtl# resource is as follows:
 * 		0 - 1	Number of entries (2 bytes)
 *		2 - 3	Entry number 1 (2 bytes)
 *		etc.
 * Sub-pane items will be overlaid atop the master dialog.
 */
DialogPtr GetNewMPDialog(short dID, Ptr dStorage, WindowPtr behind)
{
	MPDialogRecord  *mpDRec;
	Handle			handle;
	short			cnt,i;
	
	if (!dStorage) {
		mpDRec = (MPDialogRecord *) NewPtr(sizeof(MPDialogRecord));
	}
	if (!mpDRec) return nil;

	// get master dialog info
	mpDRec->dlogPtr = GetNewDialog(dID,&mpDRec->dlogRecord,behind);
	if (!mpDRec->dlogPtr) return nil;
	mpDRec->dlogID = dID;
	mpDRec->dlogItemCnt = CountDITL(mpDRec->dlogPtr);

	// get sub-pane info
	handle = Get1Resource('dtl#',dID);
	if (!handle) return nil;
	cnt = ((short*)*handle)[0];
	mpDRec->subPaneIDs = (short*)NewPtr(sizeof(short) * cnt);
	for (i=0; i<cnt; i++) {
		mpDRec->subPaneIDs[i] = ((short*)*handle)[i+1];
	}
	mpDRec->numPanes = cnt;
	mpDRec->subPaneNum  = -1;
	
	// return dialog ID.
	return (mpDRec->dlogPtr);
}

/* CloseMPDialog
 * This removes the Dialog resource and most memory w/o removing the MPDialogRecord.
 * Just what you would expect.
 */
void CloseMPDialog(DialogPtr mpDialog)
{
	if (!mpDialog) return;
	
	CloseDialog(mpDialog);
}

/* DisposeMPDialog
 * this removes the Dialog resource, and all memory associated ith it. Just what you would
 * expect it to do.
 */
void DisposeMPDialog(DialogPtr mpDialog)
{
	MPDialogRecord  *mpDRec;
	DialogPeek		dp;
	
	if (!mpDialog) return;
	
	// get MPRec Info.
	dp = (DialogPeek)mpDialog;
	mpDRec = (MPDialogRecord *) dp;
	
	// close the dialog
	CloseDialog(mpDRec->dlogPtr);
	
	// dispose the MPDialogRecord.
	DisposePtr((Ptr)mpDRec->subPaneIDs);
	DisposePtr((Ptr)mpDRec);
}

/* SetMPDialogPane
 * This routine is used to switch between panes on a dialog. First, it removes any extra
 * items added by previous calls to SetMPDialogPane. Second, it then builds the new pane.
 * paneNum is the pane number to add; it must be between 0 and the number of panes-1. If
 * paneNum is set to -1, the master dialog is returned to its original state.
 */
void SetMPDialogPane(DialogPtr mpDialog, short paneNum)
{
	MPDialogRecord  *mpDRec;
	DialogPeek		dp;
	Handle 			handle;
	short			i;
	
	if (!mpDialog) return;
	
	// get MPRec Info.
	dp = (DialogPeek)mpDialog;
	mpDRec = (MPDialogRecord *) dp;
	
	// set new paneNum
	mpDRec->subPaneNum = paneNum;
	
	// remove excess D items.
	i = CountDITL(mpDRec->dlogPtr) - mpDRec->dlogItemCnt;
	ShortenDITL(mpDRec->dlogPtr,i);
	
	// check paneNum; do we bail?
	if ((paneNum < 0) || (paneNum >= mpDRec->numPanes)) return;

	// get DITL info
	handle = Get1Resource('DITL',mpDRec->subPaneIDs[paneNum]);
	if (!handle) return;
	
	// append & cleanup.
	AppendDITL(mpDRec->dlogPtr, handle, overlayDITL);
	ReleaseResource(handle);
}

/* GetMPDialogPane
 * This routine returns the active pane number
 */
short GetMPDialogPane(DialogPtr mpDialog)
{
	MPDialogRecord  *mpDRec;
	DialogPeek		dp;
	
	if (!mpDialog) return;
	
	// get MPRec Info.
	dp = (DialogPeek)mpDialog;
	mpDRec = (MPDialogRecord *) dp;
	
	return mpDRec->subPaneNum;
}

/* MPItemHit
 * Converts the itemHit returned by ModalDialog into local subPane item numbers and
 * returns the local subPane number.
 */
void MPItemHit(DialogPtr mpDialog, short *itemHit, short *paneNum)
{
	MPDialogRecord  *mpDRec;
	DialogPeek		dp;
	
	if (!mpDialog) return;
	
	// get MPRec Info.
	dp = (DialogPeek)mpDialog;
	mpDRec = (MPDialogRecord *) dp;
	
	if (*itemHit <= mpDRec->dlogItemCnt) {
		//item is in our default pane. Leave it be.
		*paneNum = -1;
	} else {
		*itemHit -= mpDRec->dlogItemCnt;
		*paneNum = mpDRec->subPaneNum;
	}
}